import certifi
import grpc
import os

from . import helper
from ..identity.v1 import identity_management_api_pb2_grpc as pb2_grpc


class IdentityClient(object):

    def __init__(self, credential_file=None, local=False):
        if credential_file is None or credential_file is False:
            # Sometimes the env variable was set later so I have to make sure that there is no such settings
            credential_file_env = os.getenv('INDYKITE_APPLICATION_CREDENTIALS_FILE')
            try:
                if credential_file_env is False:
                    raise Exception("Missing INDYKITE_APPLICATION_CREDENTIALS_FILE environment variable")
            except Exception as exception:
                print(exception)
                return None
        else:
            credential_file_env = credential_file

        credentials = os.path.join(os.path.dirname(credential_file_env), os.path.basename(credential_file_env))
        credentials = helper.load_credentials(credentials)
        agent_token = helper.create_agent_jwt(credentials)

        call_credentials = grpc.access_token_call_credentials(agent_token.decode("utf-8"))

        if local:
            certificate_path = os.getenv('CAPEM')
            endpoint = credentials.get("local_endpoint")
        else:
            certificate_path = certifi.where()
            endpoint = credentials.get("endpoint")

        with open(certificate_path, "rb") as cert_file:
            channel_credentials = grpc.ssl_channel_credentials(cert_file.read())

        composite_credentials = grpc.composite_channel_credentials(channel_credentials,
                                                                   call_credentials)

        channel = grpc.secure_channel(endpoint, composite_credentials)
        self.stub = pb2_grpc.IdentityManagementAPIStub(channel=channel)

    # Imported methods
    from ._change_password import \
        change_password_of_user, \
        change_password
    from ._digital_twin import \
        get_digital_twin_by_token, \
        get_digital_twin
    from ._introspect_token import introspect_token
    from ._patch_properties import patch_properties_by_token, patch_properties
    from ._verification import \
        start_digital_twin_email_verification, \
        verify_digital_twin_email
